import abc
import datetime
import json

from rest_framework.views import APIView
from rest_framework.response import Response
from rest_framework.serializers import ValidationError

from django.conf import settings

from .serializers import SignalSerializer, GroupSerializer, SignalGroupSerializer, \
        RexCoreSerializer
from .authentication import IsAuthenticated


class Option(APIView):
    permission_classes = [IsAuthenticated]

    def __init__(self):
        self.option = self.define_option()

    def define_option(self):
        if hasattr(settings, 'APP_OPTION'):
            return settings.APP_OPTION
        else:
            return {}

    def get(self, request):
        return Response(self.option)


class Version(APIView):
    permission_classes = [IsAuthenticated]

    def get(self, request):
        if not hasattr(settings, 'STATIC_VERSION_FILE'):
            raise Exception("Missing version file configuration!")

        try:
            file = settings.STATIC_VERSION_FILE
            with open(file) as version_file:
                version = json.load(version_file)
        except IOError:
            raise Exception("Missing version file!")
        except Exception:
            raise Exception("Invalid version file!")

        return Response(version)


class SignalList(APIView):
    permission_classes = [IsAuthenticated]

    @abc.abstractmethod
    def get_signals(self):
        pass

    def get(self, request):
        signals = self.get_signals()
        serializer = SignalSerializer(signals, many=True)
        return Response(serializer.data)


class SignalDetail(APIView):
    permission_classes = [IsAuthenticated]

    @abc.abstractmethod
    def get_signal(self, pk):
        pass

    def get(self, request, signal_pk):
        signal = self.get_signal(signal_pk)
        serializer = SignalSerializer(signal)
        return Response(serializer.data)


class GroupList(APIView):
    permission_classes = [IsAuthenticated]

    @abc.abstractmethod
    def get_groups(self):
        pass

    def get(self, request):
        groups = self.get_groups()
        serializer = GroupSerializer(groups, many=True)
        return Response(serializer.data)


class GroupDetail(APIView):
    permission_classes = [IsAuthenticated]

    @abc.abstractmethod
    def get_group(self, pk):
        pass

    def get(self, request, group_pk):
        group = self.get_group(group_pk)
        serializer = GroupSerializer(group)
        return Response(serializer.data)


class SignalGroupsList(APIView):
    permission_classes = [IsAuthenticated]

    @abc.abstractmethod
    def get_signal_groups(self, signal_pk):
        pass

    def get(self, request, signal_pk):
        groups = self.get_signal_groups(signal_pk)
        serializer = SignalGroupSerializer(groups, many=True)
        return Response(serializer.data)


class RexCoreList(APIView):
    permission_classes = [IsAuthenticated]

    @abc.abstractmethod
    def get_connections(self):
        pass

    def get(self, request):
        groups = self.get_connections()
        serializer = RexCoreSerializer(groups, many=True)
        return Response(serializer.data)


class RexCoreDetail(APIView):
    permission_classes = [IsAuthenticated]

    @abc.abstractmethod
    def get_connection(self, pk):
        pass

    def get(self, request, group_pk):
        group = self.get_connection(group_pk)
        serializer = RexCoreSerializer(group)
        return Response(serializer.data)


class BatchSignalLastValue(APIView):
    permission_classes = [IsAuthenticated]

    @abc.abstractmethod
    def _create_rexcore_value_reader(self):
        pass

    def get(self, request):
        ids = self.request.query_params.get('ids', None)

        reader = self._create_rexcore_value_reader()
        reader.read(ids)

        response = reader.result()

        return Response(response)


class SignalDataView(APIView):
    permission_classes = [IsAuthenticated]

    def get_context(self):
        dt_start = self.request.query_params.get('dt_start', None)
        if dt_start is None:
            raise ValidationError('Field dt_start is required!')

        dt_start = datetime.datetime.strptime(dt_start, '%Y-%m-%d-%H-%M-%S') # TODO MM format
        if dt_start is None:
            raise ValidationError('dt_start has incorect format!')

        dt_end = self.request.query_params.get('dt_end', None)
        if dt_end is None:
            raise ValidationError('Field dt_end is required!')

        dt_end = datetime.datetime.strptime(dt_end, '%Y-%m-%d-%H-%M-%S') # TODO MM format
        if dt_end is None:
            raise ValidationError('dt_end has incorect format!')

        dt_diff = dt_end - dt_start
        dt_diff_seconds = dt_diff.total_seconds() # TODO MM move into helper function
        if dt_diff_seconds <= 0:
            raise ValidationError('Date range is not valid!')

        level = self.request.query_params.get('level', None)
        if level is None:
            raise ValidationError('Field level is required!')
        if not level.isnumeric():
            raise ValidationError('Level must be number!')
        level = int(level)
        if level <= 0:
            raise ValidationError('Level must be positive number!')

        window_segment = int(dt_diff_seconds / level) + 1 # TODO MM move into helper function
        if window_segment <= 0:
            raise ValidationError('Date range is not valid!')

        return {
            'dt_start': dt_start,
            'dt_end': dt_end,
            'level': level,
            'window_segment': window_segment,
        }

    @abc.abstractmethod
    def get(self, request, signal_pk):
        pass


class SignalDataAutoView(SignalDataView):
    pass


class SignalDataAnalogView(SignalDataView):
    pass


class SignalDataBarView(SignalDataView):
    pass


class SignalDataCumulativeView(SignalDataView):
    pass


class SignalDataBoolView(SignalDataView):
    pass
